{%- if current %}
{%- set current_name = current['name'] if current is mapping else current.name %}
<div class="sidebar-tree">
  <p class="caption" role="heading"><span>Versions</span></p>
  <ul>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/main/">
        <em>(latest)</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.2.0/">
        <em>v0.2.0</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.9/">
        <em>v0.1.9</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.8/">
        <em>v0.1.8</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.7/">
        <em>v0.1.7</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.6/">
        <em>v0.1.6</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.5/">
        <em>v0.1.5</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.4/">
        <em>v0.1.4</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.3/">
        <em>v0.1.3</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.2/">
        <em>v0.1.2</em>
      </a>
    </li>
    <li class="toctree-l1">
      <a class="reference" href="https://alexeev-prog.github.io/nadzoring/v0.1.1/">
        <em>v0.1.1</em>
      </a>
    </li>
  </ul>
</div>
{%- endif %}
